/******************************************************************************
File: drvinterface.h

******************************************************************************/
#ifndef DRVINTERFACE_H
#define DRVINTERFACE_H

/******************************************************************************
InitDrv: Establish connection to the driver
returns: 0 if success, 1 if connect failed
******************************************************************************/
int InitDrv(void);

/******************************************************************************
StartRead: send a message to the driver to start reading a single char to the
           address passed in pchar
returns: 0 if success in starting read, 1 if StartRead failed
******************************************************************************/
int StartRead(char *pchar);

/******************************************************************************
StartBufferRead: Send a message to the driver requesting the reception of max.
                 of size bytes placing them in the memory area pointed to by
				 pc. For each byte read from the port, a signal is received.
returns: 0 if success in starting write, 1 if StartWrite failed
******************************************************************************/
int StartBufferRead(char* pc, int size);

/******************************************************************************
StartWrite: Send a message to the driver requesting the transmission of the 
            char passed in c.
returns: 0 if success in starting write, 1 if StartWrite failed
******************************************************************************/
int StartWrite(char c);

/******************************************************************************
StartBufferWrite: Send a message to the driver requesting the transmission of a 
                  buffer, whose parameters are passed in pc and size.
returns: 0 if success in starting write, 1 if StartWrite failed
******************************************************************************/
int StartBufferWrite(char*, int);

/******************************************************************************
WaitRead:
Wait for a read to complete
******************************************************************************/
int WaitRead(void);

/******************************************************************************
WaitWrite:
Waits for a known signal from the signal handler.
******************************************************************************/
int WaitWrite(void);

/******************************************************************************
CloseDrv: Sends a message to the driver to cancel our pending reads (if any) and 
          disconnects us from the driver.
******************************************************************************/
int CloseDrv(void);


extern int int_read, int_write, int_end;
#endif





