/* File: generate.c
 * This program generates traffic for the network simulations. The data is
 * completely random and can be contaminated with noise by supplying the
 * error rate as param 4
 */
#include "layer1.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

extern formatFrame(frame_t *,char *);

void dumpmem(char* base, int size) {
	int lp1, lp2;
	
	lp2=0;
	while (lp2<size) {
		printf("%08x: [ ", base );
		for (lp1=0;lp1<16 && lp2<size;lp1++) {
			printf("%02x ", (unsigned char)*(base++) );
			lp2++;
		}
		printf("]\n");
	}
}

int rnd(int n) {
	return (int) rand()%n;
}


int main(int argc, char*argv[]) {
	frame_t msg;
	char buffer[530];
	int lp1,lp2,olen;
	FILE *f;
	int packets;
	int failrate;
	time_t ticks;
	
	time(&ticks);
	srand(ticks);
	
	if (argc<2) {
		printf("usage: generate <data file> [<frames>=16] [<fail-rate>=0]\n This program will generate data for simulation of network traffic.\n <frames> - number of frames to generate.\n <fail-rate> indicates how many bytes pr. 1000 bytes will be erroneous.");
		return 1;
	} 
	if (argc>2) packets = atoi(argv[2]); else packets=16;
	if (argc>3) failrate = atoi(argv[3]); else failrate=0;
	
	printf("%s: clearing buffer...\n",argv[0]);
	/* clear the buffer */
	for (lp1=0;lp1<sizeof(buffer);lp1++) 
		buffer[lp1]=0;
	
	printf("%s: creating file '%s'...\n",argv[0], argv[1]);
	f = fopen(argv[1],"wb");
	printf("%s: creating frames and writing to file...\n",argv[0]);
	for (lp1=0; lp1<packets;lp1++) {
		/* generate a frame */
		msg.preamble=PREAMBLE;
		msg.source=rnd(15)+1;
		msg.dest=rnd(16);
		msg.ack=rnd(2);
		msg.sn=rnd(8);
		msg.TTL=rnd(15);
		msg.size=rnd(256);
		for (lp2=0;lp2<msg.size;lp2++)
			msg.data[lp2]=lp2&0x7f;
		/* translate the frame to network format */
		olen = formatFrame(&msg,buffer);
		printf("FRAME: %02x{ from=%d, to=%d, acked=%d, sn=%d, TTL=%d, size=%d, checksum=%02x }\n",msg.preamble, msg.source, msg.dest, msg.ack, msg.sn, msg.TTL, msg.size, msg.checksum);
		/* contaminate data with noise! */
		for (lp2=0;lp2<olen;lp2++) 
			if (rnd(1000)<failrate) {
			buffer[rnd(olen)]=rnd(255);
		}
		dumpmem(buffer,olen);
		
		fwrite(buffer,olen,1,f);
	}
	/* generate a frame 
		msg.preamble=PREAMBLE;
		msg.source=8;
		msg.dest=1;
		msg.ack=0;
		msg.sn=2;
		msg.TTL=3;
		msg.size=230;
		for (lp2=0;lp2<msg.size;lp2++)
			msg.data[lp2]=lp2&0x7f;
		olen = formatFrame(&msg,buffer);
		printf("FRAME: %02x{ from=%d, to=%d, acked=%d, sn=%d, TTL=%d, size=%d, checksum=%02x }\n",msg.preamble, msg.source, msg.dest, msg.ack, msg.sn, msg.TTL, msg.size, msg.checksum);
		fwrite(buffer,olen,1,f);
	*/

	fclose(f);
	return 0;
}
