/* File: simulate.c
 * This program runs a simulation of the network on layer1. It fakes the real
 * network by using a data-in file.
 */
#include "layer1.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

void dumpmem(char* base, int size) {
	int lp1, lp2;
	
	lp2=0;
	while (lp2<size) {
		printf("%08x: [ ", base );
		for (lp1=0;lp1<16 && lp2<size;lp1++) {
			printf("%02x ", (unsigned char)*(base++) );
			lp2++;
		}
		printf("]\n");
	}
}

extern frame_t forward_buffer[FORWARD_BUFFER_SIZE];
extern int forward_inpos;
extern int forward_outpos;

int main(int argc, char* argv[]) {
	FILE *f, *fout;
	frame_t msg;
	int c;
	int lp1,e,len;
	char outdata[550];
	int addr;
	int writeoutfile;

	if (argc<2) {
		printf("usage: simulate <source data file> [<addr>=1 [<outfile>]]\n Does simulation of network by reading data from file...",argv[0]);
		return 1;
	}
	/* read address from args */
	if (argc>=3) addr=atoi(argv[2]);
	else addr=1;
	if (addr<1) addr=1;
	if (addr>15) addr=15;

	/* save simulated input */
	writeoutfile=0;
	if (argc>=4) {
		fout = fopen(argv[3],"wb");
		if (!fout) {
			printf("%s: failed opening the file '%s'!\n",argv[0], argv[3]);
			return 3;
			writeoutfile=0;
		}
		writeoutfile=1;
	}

	lp1=0;
	f = fopen(argv[1],"rb");
	if (!f) {
		printf("%s: failed opening the file '%s'!\n",argv[0], argv[1]);
		return 2;
	}
	
	init(addr);
	
	/* now simulate a read of this frame... */
	printf("%s: reading network data...\n",argv[0]);
	
	while (!feof(f)) {
		c=getc(f);
		if ( !feof(f) ) onReadChar(c);
		if ( hasInFrames() ) {
			if (e=getFrame(&msg))
				printf("couldn't read frame from in-buffer even though it should be there...%d\n",e);
			else
				printf("getFrame: %02x{ from=%d, to=%d, acked=%d, sn=%d, TTL=%d, size=%d, checksum=%02x } data:\n",msg.preamble, msg.source, msg.dest, msg.ack, msg.sn, msg.TTL, msg.size, msg.checksum);
			dumpmem(msg.data, msg.size);
			frameDone=0;
		}
		if ( len=prepareForward(outdata) ) {
/*			dumpmem(outdata,len);
*/			if (writeoutfile) fwrite(outdata, len, 1, fout);
			printf("simulate: forwarded %d bytes...", len);
			printf("\n--------------------\n");
			lp1++;
		}
	}
	
	if (writeoutfile) fclose(fout);
	/* dump the entire forward buffer state */
	printf("forwarded %i frames\n", lp1);
	printf("FORWARD BUFFER: forward_inpos=%d, forward_outpos=%d\n", forward_inpos, forward_outpos);
	for (lp1=0; lp1<FORWARD_BUFFER_SIZE; lp1++) {
		msg=forward_buffer[lp1];
		printf("FRAME: %02x{ from=%d, to=%d, acked=%d, sn=%d, TTL=%d, size=%d, checksum=%02x }:\n",msg.preamble, msg.source, msg.dest, msg.ack, msg.sn, msg.TTL, msg.size, msg.checksum);
	}
		
	return 0;
}
