/* File: layer2.h
 *
 * File contains interface towards network driver (shared dm and pipe names)
 * A process that needs to send a packet locks sendpipe_sem, transmits the std.
 * head for a packet and the data and releases the sendpipe.
 * The ONE receiver locks the recvpipe_sem and listens for packets from the
 * network.
 *
 * WARNINGS:
 *  - AT MOST 254 BYTES MAY BE SENDT AT ONCE!!!
 *  - YOU CANNOT DETERMINE WHAT PROCESS WAS THE RECEIVER. ALL DATA THE SAME.
 *  - REMEBER TO CLEAR THE RECEIVE PIPE (networker may lock if pipe not cleared)
 */
#ifndef LAYER2_H
#define LAYER2_H

#include <types.h>
#include <semaphore.h>

#define DATA_MODULE_NAME "net_dm"
#define SEND_PIPE_NAME "/pipe/net_send"
#define RECV_PIPE_NAME "/pipe/net_recv"
#define PACKETHEADSIGNATURE 0xff;

/* This is the structure of the data in the shared data module. The semaphores
 * are used to control the exclusive use of the pipes. When sending a packet
 * on the net, the sendpipe is locked while sending the header and the data.
 * The application sending a packet will receive the signal supplied when the
 * packets destiny is known.
 * When an application is using the recvpipe, it needs to lock the semaphore
 * and set up a signal for data-in-pipe. Data will be in same format as 
 * in the transmit pipe.
 */
typedef struct {
	/* lock this semaphore while using the sendpipe to ensure that packets
	 * are complete in the pipe */
	semaphore sendpipe_sem;
	/* lock this semaphore when using the receive pipe - one user only */
	semaphore recvpipe_sem;
	/* set this to 1 if you are listening. 
	 * set this to 0 when you stop listening.
	 * Protected by semaphore recvpipe_sem. */
	char recv_listen;
	/* name of the pipe for transmitting packets on the network */
	char sendpipe[64];
	/* name of the pipe to receive packets from the network */
	char recvpipe[64];
} l2_data_t;

/* This is the packet header. It should be written to the sendpipe with the data
 * immediately following.  
 */
typedef struct {
	char signature; /* =0xff */
	process_id process; /* your process ID */
	signal_code sigack; /* signal to receive when packet delivery succeded */
	signal_code signak; /* signal to receive when packet delivery failed */

	unsigned dest:4; /* destination for packets */
	int32 timeout; /* time-out in ticks - currently ignored */
	int32 size; /* AT MOST 254 BYTES! MORE WILL BE TRUNCATED AND SEVERLY MESS UP THE SYSTEM! */
	/* char data[packetsize]; AT MOST 254 BYTES! MORE WILL MESS UP NETWORKING!!! */
} sendpacket_t;

/* This is the packet header. It should be written to the recvpipe with the data
 * immediately following.  
 */
typedef struct {
	char signature; /* =0xff */
	unsigned source:4; /* source of packets */
	unsigned dest:4; /* destination for packets */
	u_char size; /* AT MOST 254 BYTES! MORE WILL BE TRUNCATED AND SEVERLY MESS UP THE SYSTEM! */
	/* char data[packetsize]; AT MOST 254 BYTES! MORE WILL MESS UP NETWORKING!!! */
} recvpacket_t;


#endif
