/* File: layer2_proto.h
 *
 * These are the prototypes for layer-2. Not included in layer2.h because
 * other processes may include layer2.h
 */
#ifndef LAYER2_PROTO_H
#define LAYER2_PROTO_H

/* make sure that out_heads only contain invalid process ID's 
 * return: E_OK
 */
int l2_init(void);

/* Called when a new frame is present in the in-buffer. Will
 * decrement in_changed if it manages to write the frame data to the recvpipe.
 * If it doesn't write the frame received, it will be called on next frame recv.
 * and try again.
 * return: E_OK
 *         E_PIPEOVERFLOW
 */
int l2_handle_inframe(void);

/* Whenever out_states are updated, this function is called and will send 
 * signals to all affected processes.
 * return: E_OK
 */
int l2_handle_replies(void);

/* When data is available in the sendpipe, this function is called and will
 * read data from the sendpipe and add to outbuffer.
 * return: E_OK - when packet is added to outbuffer
 *         E_NOFRAMESAVAIL - when no free buffers where found
 *         E_NOTREADY - pipe data too small
 */
int l2_handle_new(void);

/* Retires packet with time-outs, messes with out_buffer, out_states
 * and out_changed directly... It is assumed, that this method is called
 * at least 2 times a day (otherwise it will miscalculate time passed and
 * severly mess up).
 * param :
 * return: E_OK
 */
int l2_ding(void);

#include <types.h>
#include "layer2.h"
#define RECV_PIPE_SIZE 2048
#define SEND_PIPE_SIZE 2048

/* vars needed for communicating on level2 */
extern l2_data_t *dm;
extern path_id sendpipe;
extern path_id recvpipe;

#endif
