/* File: write.c
 * Simple program to write a single character to the cereal port.
 */
 
#include <types.h>
#include <process.h>
#include <sysio.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

/* Mem I/O of serial port 2 (B) */
u_int8 *portBreg = (void*)0xfffe3000; /* normal access - reg0 derefter register */
u_int8 *portBio = (void*)0xfffe3001; /* direkte til register 8 */
/* Mem I/O of serial port 1 (A) */
u_int8 *portAreg = (void*)0xfffe3002;
u_int8 *portAio = (void*)0xfffe3003;


int initPort(void);
int closePort(void);
int sendChar(char);


int main(int argc, char* argv[]) {
	int no_runs=0;
	int lp1=0;
	u_int32 ticks;
	
	if (argc<2) {
		printf("Usage: %s <number-of-chars-to-write> (default: 1)\n", argv[0]);
		no_runs=1;
	}	
	else {
		no_runs= atoi(argv[1]);
	}
	
	printf("%s: started, getting permissions...\n",argv[0]);
	if (_os_permit(portBreg, 2, READ_|WRITE_, 0)) {
		printf("%s: failed getting permissions\n",argv[0]);
		_os_exit(1);
	}
	
	printf("%s: initing port...\n",argv[0]);
	initPort();
	printf("%s: sending %i char(s)...\n",argv[0], no_runs);
	for (lp1=0;lp1<no_runs;lp1++) {
		sendChar( 'a'+(lp1%25) );
		ticks=10;
		_os9_sleep(&ticks);
	}
	printf("%s: closing port...\n",argv[0]);
	closePort();
	
	_os_exit(0);
}

int initPort() {
	/* reset SCC */
	/**portBreg=0x00;*/

	/* Reset in WR9 (Master Int. Control)
	 * ChannelB reset,  [7:6]=01
	 * Software int. ack, [5]=0 reserved on NMOS
	 * Status High/Low, [4]= 0
	 * Resten = 0 (naar int. er slaaet fra)
	 */
	*portBreg=0x09; *portBreg=0x40;
	
	
	/* set WR4 (transmit/receive modes) to:
	 * clock mode - 16x, [7:6]=01
	 * sync char - 8-bit, [5:4]=00  *ignored i asyncmode
	 * stop bits - 1, [3:2]=01
	 * parity - disabled [1:0]=00
	 */
	*portBreg=0x04; *portBreg=0x44;
	
	/* set WR3 (receive params) to:
	 * bits per char - 8, [7:6]=11
	 * auto enable - disabled, [5]=0
	 * hunt mode - disabled, [4]=0   *ignored i asyncmode
	 * receive CRC - disabled, [3]=0   *ignored i asyncmode
	 * address search mode - disabled, [2]=0   *ignored i asyncmode
	 * sync char load inhibit - disabled, [1]=0
	 * receiver enabled - disabled, [0]=0
	 */
	*portBreg=0x03; *portBreg=0xC0;
	 
	/* set WR5 (transmit params) to:
	 * DTR control - enabled, [7]=1
	 * char select bits - 8, [6:5]=11
	 * send break - disabled, [4]=0
	 * transmit enabled - disabled, [3]=0 * while initing
	 * SDLC/CRC16 - unset, [2]=0  *ignored i asyncmode
	 * RTS control - disabled, [1]=0
	 * transmit CRC enable - disabled, [0]=0  *ignored i asyncmode
	 */
	*portBreg=0x05; *portBreg=0xE0;
	
	/* set WR1 (interrupt and data transfer):
	 * wait/dma request enable - , [7]=0
	 * wait/dma request func. - , [6]=0
	 * wait/dma request on recv/trans - , [5]=0
	 * Rx int. - on all chars and spec. cond., [4:3]=10
	 * parity is special condition - no, [2]=0
	 * Tx int enable - yes, [1]=1
	 * ext. int. enable - no, [0]=0
	 */
/*	*portBreg=0x01; *portBreg=0x12; */
	 
	/* set WR2 (interrupt vector)
	 * vector -  88_dec, [7:0]=01011000
	 */
/*	*portBreg=0x02; *portBreg=0x58; */

	/* set WR9 (master interrupt control)
	 */
	 
	/* set WR10 (recv./trans. control bits)=0
	 * CRC preset - [7]=0  *ignored i asyncmode
	 * data enc. - NRZ, [6:5]=00
	 * go-active-on-poll - 0, [4]=0  *ignored i asyncmode
	 * mark flag idle ctrl - 0, [3]=0   *ignored i asyncmode
	 * abort/flag on underrun - 0, [2]=0   *ignored i asyncmode
	 * loop mode - disabled, [1]=0   *ignored i asyncmode
	 * sync select, [0]=0   *ignored i asyncmode
	 */
	 *portBreg=0x0A; *portBreg=0x00;
	 
	 /* set WR11 (clock mode control)
	  * BR generator (not nescessary)
	  */
/*	 *portBreg=0x0B; *portBreg=0x; */
	  
	/* set WR12,WR13 (baud rate control)
	 * time constant = 14
	 * WR12=14
	 * WR13=0
	 */
	 *portBreg=0x0D; *portBreg=0x00; 
	 *portBreg=0x0C; *portBreg=0x0E;
	
	/* set WR14 (misc. control bits
	 * dig. phase-locked loop ctrl - null command, [7:5]=000
	 * local loop-back enable - no, [4]=0
	 * auto echo - no, [3]=0
	 * DTR... - [2]=0
	 * BR generator source - , [1]=1
	 * BR generator enable - yes, [0]=1
	 */
	*portBreg=0x0E; *portBreg=0x03;
	
	/* WR15 irrelevant */
	
	/* enable receiver in WR3 */
	
	/* WR5[3] = Tx enable */
	*portBreg=0x05; *portBreg=0xE8;

	return 0;
}



/* WR3[0] = Rx enable */

int closePort() {
	/* disable all interrupts related to serial port */

	return 0;
}


int sendChar(char c) {
	/* send char to on-chip FIFO */
	*portBio=c;
/*	*portBreg=0x08; *portBreg=c; */

	

	return 0;
}
