/******************************************************************************
File: interface.h

Exercise 3.3: Prototypes for MVME147 timer 2 device driver interface library.

By using the functions defined in this file it is possible to control the
device driver without any knowledge of its inner workings.
Use the functions by including this file and link the library 'interface.l' to
your program.
******************************************************************************/
#ifndef INTERFACE_H
#define INTERFACE_H


/******************************************************************************
InitDrv:
This function initializes the timer 2 device driver. Call this function after
setting up the timer and prior to waiting for the interrupt with the
'WaitInt()' function.
Parameter:
  *irqsrv(): Pointer to the interrupt service function that is called whenever
             the timer interrupt occurs. The service function is executed in
             supervisor state from the driver. This means that it cannot share
             variables with the main program. It must be self contained.
Returns:
  0: if the timer driver was initialized without errors.
  1: if the program cannot access the timer driver.
  2: if the device driver connection failed.
******************************************************************************/
int InitDrv(void (*irqsrv)(void));


/******************************************************************************
WaitInt:
This function waits for a timer interrupt to occur. It can be aborted by
pressing Ctrl+C or Ctrl+E.
'InitDrv()' must be called prior to this function. The interrupt and timer
must be enabled also.
Returns:
  0: if a timer interrupt occured.
  1: if the user pressed Ctrl+C to abort the program.
  2: if the device driver connection failed.
******************************************************************************/
int WaitInt(void);


/******************************************************************************
StopDrv:
Releases the connection to the timer device driver established with the
'InitDrv()' function.
Returns:
  0: if no problems occured.
  1: if the device driver connection release failed.
******************************************************************************/
int StopDrv(void);

#endif
