/* timer.c
 * Program outputs a string to console at given intervals using interrupts
 *
 * Version: 1.0: Initial Release
 * Authors: Mikael Lund Jepsen (c973811) and Rune M. Barnkob (c973431)
 */
 
#include <stdio.h>
#include <process.h>
#include <sysio.h>
#include "interface.h"

/*void timerint(void);
int main(void);*/

typedef unsigned short uint16;
typedef unsigned char uint8;

void timerint(void) {
uint8 *intctrl= (void*)0xFFFE101A;
	/* reset interrupt */
	*intctrl = 0x8E; 						/* 1000 1110 Intstat, enable, intlevel 6 */
	return;
}

int main(void) {
	uint16 *preload= (void*)0xFFFE1014;
	uint8 *intctrl= (void*)0xFFFE101A;
	uint8 *ctrl= (void*)0xFFFE101B;

	int retcode=0, lp1=0, lp2=0;
	printf("TIMER\n-----\n");

	printf("Requesting memory access to control regs...\n");
	/* request access to timer2's control regs */
	
	if ( _os_permit((void*)intctrl, 1, WRITE_,0) || 
	     _os_permit((void*)ctrl, 1, WRITE_,0) || 
		 _os_permit((void*)preload, 2, WRITE_,0) ) {
		printf("Failed to get access to Timer2's memory mapped IO\n");
		return 1;
	}

	printf("done.\n");
	
	printf("Setting up timer...");	
	/* setup timer */
	*preload = -16000;					/* generate 16000 tics before interrupt */
	*ctrl = 0x04;						/* load preload, clrovf, disable */
	*intctrl = 0x8E;					/* 1000 1110 generate int, intlevel 6 */
	printf("done.\n");
	
	printf("Attempting to start driver...");
	/* Connect to interruptdriver */
	retcode = InitDrv(timerint);
	if (retcode) {
		printf("Timer initialisation error, code:%i", retcode);
	}
	else {
		printf("done.\n");

		/* Start timer */
		printf("Starting timer...");
		*ctrl = 0x07; 						/* 0000 0111 clrovf, enable, enable */
		printf("done.\n");
		for(lp1=0; lp1<5; lp1++)			/* Run a number of times */
		{
			for (lp2=0; lp2<50; lp2++) {	/* wait until for 50*100 msec. */
/*				printf("Entering wait... \n"); */
				retcode = WaitInt();
/*				printf("done\n"); */
				if (retcode) {
					printf("Timer interrupt error, code:%i", retcode);
				}
			}
			printf("message occurred!\n");
		}
	
		/* Cleanup before exit */
		printf("Cleaning up and exiting...");
		/* Stop timer */
		*ctrl = 0x04; /* load preload, clrovf, disable */
		*intctrl = 0x86;					/* 1000 0110 generate int, intlevel 6 */		
		
		/* Stop the driver and prepare to exit */
		StopDrv();		/* Disconnect from interruptdriver */
		printf("done.\n");
	}
	return retcode;	
}
