/* freely based upon the matrix library given by IAU
 * this code is released under the GNU public license (even though we have no clue of the implications!)
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <os9timing.h>	 
#include <iau_mat.h>

/* Empty subroutine used to measure */
void emmul( matrix *ptm1, matrix *ptm2, matrix *ptm3 )
{
	register int i,j;

}

/* Naive matrix multiplication using nested for-loops (ptm1 = ptm2 * ptm3) */
void immul( matrix *ptm1, matrix *ptm2, matrix *ptm3 )
{
	register int r,c,lp1;
	/* check matrix dimensions */
	/*if (ptm2->col==ptm3->row && ptm1->row==ptm2->row && ptm1->col==ptm3->col) */ {
		/* Multiply the two matrices */
		for (r=0; r<ptm1->row; r++)
			for (c=0; c<ptm1->col; c++) {
				ptm1->mat[r][c] = 0;
				for (lp1=0; lp1<ptm2->col; lp1++)
					ptm1->mat[r][c] += ptm2->mat[r][lp1]*ptm3->mat[lp1][c];
			}
	}
}

/* calculation removed from loop to measure loop overhead */
void iemmul( matrix *ptm1, matrix *ptm2, matrix *ptm3 )
{
	register int r,c,lp1;

	/* Add the two matrices element by element */
	/* check matrix dimensions */
	/*if (ptm2->col==ptm3->row && ptm1->row==ptm2->row && ptm1->col==ptm3->col) */ {
		/* Multiply the two matrices */
		for (r=0; r<ptm1->row; r++)
			for (c=0; c<ptm1->col; c++) {
				ptm1->mat[r][c] = 0;
				for (lp1=0; lp1<ptm2->col; lp1++);
			}
	}
}

void pmmul( matrix *ptm1, matrix *ptm2, matrix *ptm3 )
{
	register double *a,*b,*b_rend,*b_mend, *d, *dend;

	register int bstep;
	/* d points to destination for result in ptm1,
	 * c points to last element in ptm1
	 * a used for fast summation of cols in ptm2
	 * b used for fast summation of rows in ptm3
	 */
	 
	/*if (ptm2->col==ptm3->row && ptm1->row==ptm2->row && ptm1->col==ptm3->col) */ {
		/* Multiply the two matrices */
		d = ptm1->mat[0];
		dend = d+ptm1->col*ptm1->row;
		a = ptm2->mat[0];
		b = ptm3->mat[0];
		bstep = ptm3->col;
		b_mend = b+ptm3->col*ptm3->row;
		b_rend = b+bstep;
		
		
		/* for each element in result do */
		while (d<dend) {
			while (b<b_rend) {
				*d = 0;
				while (b<b_mend) {
					*d += (*a)*(*b);
					a++;
					b+=bstep;
				}
				b = b-bstep*(ptm3->row)+1;
				a -= ptm2->col;
				d++;
			}
			a += ptm2->col;
			b -= bstep;
		}
		
	}					
}

void ammul( matrix *ptm1, matrix *ptm2, matrix *ptm3 )
{
/*
a0=d
a1=d_matrixend
a2=a=ptm2->mat[0]
a3=b=ptm3->mat[0]
a4=b_matrixend
a5=b_rowend
d5=a_cols, number of bytes
d6=b_cols, number of bytes to move b forward with
d7=b_rows
*/
_asm("
	movem.l	d0-d7/a0-a5,-(sp)
	
	move.l	%5,a5
	move.l	%6,d6
	move.l	%0,a0
	move.l	%8,d5
	move.l	%4,a4
	move.l	%2,a2	
	move.l	%7,d7	
	move.l	%3,a3
	move.l	%1,a1
	
	mulu.l	#8,d6
	mulu.l	#8,d5
	
while1:
	cmp.l	a1,a0
	bge.s	while1end
	
while2:
	cmp.l	a5,a3
	bge.s	while2end
	fmove.d	#0,fp0
	
while3:
	cmp.l	a4,a3
	bge.s	while3end
	fmove.d	(a2)+,fp1
	fmul.d	(a3),fp1
	fadd	fp1,fp0
	add.l	d6,a3
	bra.s	while3
while3end:
	fmove.d	fp0,(a0)+
	move.l	d6,d0
	mulu.l	d7,d0
	sub.l	d0,a3
	add.l	#8,a3
	sub.l	d5,a2
	bra.s	while2
	
while2end:
	add.l	d5,a2
	sub.l	d6,a3
	bra.s	while1
	
while1end:
	
	movem.l	(sp)+,d0-d7/a0-a5
	
	",
		ptm1->mat[0],
		ptm1->mat[0]+ptm1->col*ptm1->row,
		ptm2->mat[0],
		ptm3->mat[0],
		ptm3->mat[0]+ptm3->col*ptm3->row,
		ptm3->mat[0]+ptm3->col,
		ptm3->col,
		ptm3->row,
		ptm2->col
);		
}

void main(int argc, char *argv[])
{
	matrix *a,*b,*c;
	int i, matsize, no_iter;
	unsigned int t1,t2;
	double res;
	
	if (argc == 3) {
		matsize = atoi(argv[1]);
		no_iter = atoi(argv[2]);
	} else {
		printf("Usage: %s matsize no_iter\n",argv[0]);
		exit(1);
	}
	/* Allocate matrixes a,b and c  */ 
	a=mmake(matsize,matsize); b=mmake(matsize,matsize); c=mmake(matsize,matsize);
	/* Initialize a and b with random numbers */
	mrand(a); mrand(b);
	/* Open and reset timer */
/*	printf("\na=\n");
	mprint(a);
	printf("\nb=\n");
	mprint(b);
	immul(c,a,b);
	printf("\nc=\n");
	mprint(c);
	ammul(c,a,b);
	printf("\npc=\n");
	mprint(c);
*/	
	opentim();
	resettim();
	for (i = 0; i < no_iter; i++) {
		emmul(c,a,b); 
	}
	t1 = READTIM;
	printf("The empty routine test emmul:	 %lg us", t1*usecs_per_tick()/no_iter);
	printf("\n");

	resettim();
	for (i = 0; i < no_iter; i++) {
		iemmul(c,a,b); 
	}
	t1 = READTIM;
	printf("The empty loop test iemmul:	 %lg us", t1*usecs_per_tick()/no_iter);
	printf("\n");

	resettim();
	for (i = 0; i < no_iter; i++) {
		immul(c,a,b); 
	}
	t1 = READTIM;
	printf("The index based test immul:	 %lg us", t1*usecs_per_tick()/no_iter);
	printf("\n");

	resettim();
	for (i = 0; i < no_iter; i++) {
		pmmul(c,a,b); 
	}
	t1 = READTIM;
	printf("The pointer based test pmmul:	 %lg us", t1*usecs_per_tick()/no_iter);
	printf("\n");

	resettim();
	for (i = 0; i < no_iter; i++) {
		ammul(c,a,b); 
	}
	t1 = READTIM;
	printf("The assembler based test ammul:	 %lg us", t1*usecs_per_tick()/no_iter);
	printf("\n");

/*	mprint(c);
	mprint(a);
	mprint(b);	*/
	
	mfree(a); mfree(b); mfree(c);

}
