/* clocksrv.c
 * This is the clock server for the pipe-version of the pipe system
 * It writes the current time to a pipe and sleeps for n secs.
 */
#include <stdio.h>
#include <process.h>
#include <time.h>
#include <alarm.h>
#include <signal.h>
#include <types.h>
#include <stdlib.h>
#include <dexec.h>
#include <modes.h>
#include <sysio.h>
#include <cglob.h>

#define WAKEUP_CODE 1 /* The wake up signal code is 1 in OS-9 */
/* Macro for testing return value of fct. and cause exit */
#define TEST(a,start,end,err) printf(start); \
	if (a) { printf(err); return 0; } \
	else printf(end);
#define __TEST(a,start,end,err) a;
#define _TEST(a,start,end,err) if (a) { printf(err); return 1; }

struct gtime_t { 
	u_int8 reserved;
	u_int8 hour;
	u_int8 min;
	u_int8 sec; 
};

struct gdate_t { 
	u_int16 year; 
	u_int8 month;
	u_int8 day;
};

int main(int argc, char *argv[]) {
	alarm_id id;
	u_int32 ticks;
	u_int16 day;
	u_int32 gtticks;
	struct gtime_t gtime;
	struct gdate_t gdate;
	char buffer[32];
	u_int32 count, strlen;
	path_id pathid;
	char pipename[] = "/pipe/h6clock";
	int runs;
	char *argchild[3];
	process_id childid;
	u_int16 type_lang;
	char orphan = 0;


	argchild[0] = "clockcli";
	argchild[1] = pipename;
	argchild[2] = NULL;
	
	if (argc<2) {
		printf("usage: %s <number-of-runs>\nRuns and writes to pipe	/pipe/h6clock until number-of-runs\n",argv[0]);
		_os_exit(1);
	} else {
		runs = atoi(argv[1]);
	}
	
	TEST(_os9_create( pipename,WRITE_,&pathid,0777),
		"_os9_create(): creating pipe...",
		"success\n",
		"failed, exiting\n");
		
	if (!_os_alarm_cycle( &id, WAKEUP_CODE, CLOCKS_PER_SEC )) {
		printf("os_alarm installed sucesfully...\n");
		if (!_os_exec(_os_fork, 0, 3, argchild[0], argchild, _environ, 0, &childid, type_lang, orphan )) {
								/* priority=0, pathcnt=4, modname, arguments, envvars, stacksize=0(default) */
			printf("os_exec ok...\n");
			for (;runs>0;runs--) {
				ticks=0;
				TEST(_os9_sleep(&ticks),
					"Entering sleep...",
					"done sleeping\n",
					"sleep failed\n");
				
				TEST(_os9_getime(0, (u_int32*)&gtime, (u_int32*)&gdate, &day, &gtticks ),
					"Getting time...\n",
					"done\n",
					"failed\n");
				
				strlen=sprintf(buffer,"%04d-%02d-%02d %02d:%02d:%02d\n",gdate.year,gdate.month,gdate.day,gtime.hour,gtime.min,gtime.sec);
				count = sizeof(strlen);
				if (_os_write(pathid,&strlen,&count) || _os_write(pathid,buffer,&strlen))	{
					break; /* write to pipe failed, so exiting writeloop */
				}
			}
			/* ask childprocess to exit */
			count=sizeof(strlen);
			strlen=5;
			_os_write(pathid, &strlen, &count);
			_os_write(pathid,"quit",&strlen);
		}
	}	
		
	TEST(_os_close(pathid),
		"_os_close(): Closing pipe...",
		"done\n",
		"failure\n");
	return 0;
}
