/* File: dmwrite.c
 *
 * This program allocates a memory area, writes a date to it, spawns dmread, 
 * waits for it to exit, deallocates the memory area and exits.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <types.h>
#include <time.h>
#include <cglob.h>
#include <sysio.h>
#include <semaphore.h>
#include <signal.h>
#include <alarm.h>
#include "dmclock.h"

struct gtime_t { 
	u_int8 reserved;
	u_int8 hour;
	u_int8 min;
	u_int8 sec; 
};

struct gdate_t { 
	u_int16 year; 
	u_int8 month;
	u_int8 day;
};

#define WAKEUP_CODE 1

int main(int argc, char *argv[]) {
	u_int32 ticks;
	u_int16 day;
	u_int32 gtticks;
	struct gtime_t gtime;
	struct gdate_t gdate;
	char *argchild[3];
	process_id childid;
	u_int16 attr_rev, type_lang;
	char orphan = 0;
	status_code childerr;
	char *memname = "h6data";
	u_int32 memsize = sizeof(struct mdate_t);
	struct mdate_t *date;
	mh_data *modhead;
	int n;
	alarm_id aid;

	
	if (argc<2 || (n=atoi(argv[1]))<=0 ) {
		printf("Usage: %s <number-of-runs>\n", argv[0]);
		_os_exit(1);
	}
	
	printf("%s: starting...\n", argv[0]);
	
	argchild[0] = "dmread";
	argchild[1] = memname;
	argchild[2] = NULL;
	
	/* allocate memory area */
	attr_rev = mkattrevs(MA_REENT, 0);
	type_lang = mktypelang(MT_ANY, ML_ANY);
	if (!_os_datmod(memname, memsize, &attr_rev, &type_lang, MP_OWNER_READ | MP_OWNER_WRITE, (void **)&date, &modhead)) {
		_os_sema_init( &(date->mem) );
		_os_sema_init( &(date->next) );
		_os_alarm_cycle( &aid, WAKEUP_CODE, CLOCKS_PER_SEC);
		/* spawn child process */
		type_lang = mktypelang(MT_PROGRAM, ML_OBJECT);
		if (!_os_exec(_os_fork, 0, 3, argchild[0], argchild, _environ, 0, &childid, type_lang, orphan )) {
								/* priority=0, pathcnt=4, modname, arguments, envvars, stacksize=0(default) */
			printf("starting child %s, wating to end\n", argchild[0]);								
			printf("%s: created %s\n", argv[0], memname);
			for (;n--;) { /* not to be mistaken for a while construction!! */
				/* get time */
				_os9_getime(0, (u_int32*)&gtime, (u_int32*)&gdate, &day, &gtticks );
				/* write to memory area */
				_os_sema_p( &(date->mem) );
				sprintf(date->date,"%04d-%02d-%02d %02d:%02d:%02d\n",gdate.year,gdate.month,gdate.day,gtime.hour,gtime.min,gtime.sec);
				date->exit=0;
				_os_sema_v( &(date->mem) );
				/* make client sleep for period */
				ticks=0;
				_os9_sleep(&ticks);
				_os_sema_v( &(date->next) );
			}
			_os_sema_p( &(date->mem) );
			date->exit=1;
			_os_sema_v( &(date->mem) );
			_os_sema_v( &(date->next) );
			/* wait for child to exit */
			_os_wait(&childid, &childerr);
		}
		/* deallocate memory area */
		_os_unlink((mh_com*)modhead);
	} else {
		printf("%s: allocating memory failed\n",argv[0]);
	}
	/* exit */
	printf("%s: exiting\n", argv[0]);
	_os_exit(0);
}
