/******************************************************************************
File: netdefs.h

Exercise 4.3: Common definitions for the network programs.

The following macros may not be changed or the transmitter program might not
work properly: TRANS_PIPE_NAME, END_OF_TRANS, MAX_MSG_LENGTH.
******************************************************************************/

#ifndef _NETDEFS_H
#define _NETDEFS_H

#include <semaphore.h>

#define TRUE             1
#define FALSE            0

/* Name of the transmitter pipe. May not be changed. */
#define TRANS_PIPE_NAME  "/pipe/trans_pipe"

/* Name of the pipes to the writer processes. X must be replaced by the pipe */
/* number so e.g. writer 3 uses the '/pipe/writer_pipe_3' pipe.              */
#define WRITER_PIPE_NAME "/pipe/writer_pipe_%d"

#define END_OF_TRANS     "EOT"  /* The end of transmission message */
#define MAX_PROCS        8      /* Maximum number of writer processes */

#define BUF_MOD_NAME     "bufmod"  /* Name of data module */
#define NR_OF_BUFFERS    2         /* Number of message buffers */
#define MAX_MSG_LENGTH   80        /* Maximum length of messages */

/* The data module structure. */
typedef struct
{
  u_int32 time_stamp[NR_OF_BUFFERS]; /* Time stamps for messages */
  char msg_buf[NR_OF_BUFFERS][MAX_MSG_LENGTH];  /* Message buffers */
  int buf_used[NR_OF_BUFFERS];       /* Flags indicating if a buffer is used */
  semaphore buf_sema[NR_OF_BUFFERS]; /* Semaphores to protect buffer flags */
} BufModType;

#endif
