/* File: netstart.c
 *
 * creates all resources and starts sub-processes
 */

#include <stdio.h>
#include <process.h>
#include <time.h>
#include <alarm.h>
#include <signal.h>
#include <types.h>
#include <stdlib.h>
#include <dexec.h>
#include <modes.h>
#include <sysio.h>
#include <cglob.h>
#include <string.h>
#include "netdefs.h"

int main(int argc, char* argv[]) {
	char pipename[64];
	int nofwri=0;
	u_int32 count;
	u_int32 dest;
	/* Data module vars */
	u_int16 attr_rev, type_lang;
	mh_data* dm_head;
	char dm_name[] = BUF_MOD_NAME;
	char* dm_namep=dm_name;
	BufModType *dm;
	/* writer pipes */
	path_id wpipes[MAX_PROCS];
	int nofpipes;
	/* transmitter pipe */
	path_id tpipe;
	/* process ids */
	process_id pids[MAX_PROCS+2];
	u_int32 nofprc=0;
	/* child process spawning */
	char* argchild[5];
	char orphan;
	status_code retcode;
	/* running ressources */
	char ok_dm = 0;
	char ok_tpipe = 0;
	char ok_wpipes = 0;
	char ok_writers = 0;
	char ok_receiver = 0;
	char ok_transmitter = 0;
	char ok_sema = 0;
	
	/* check args */
	if (argc<2) {
		printf("usage %s <no-of-writers>\n", argv[0]);
		_os_exit(1);
	}
	nofwri=atoi(argv[1]);
	
	/* create data module */
	printf("%s: creating shared memory %s...\n",argv[0],dm_namep);
	attr_rev = mkattrevs(MA_REENT, 0);
	type_lang = mktypelang(MT_ANY, ML_ANY);
	if (_os_datmod(dm_namep, sizeof(BufModType), &attr_rev, &type_lang, MP_OWNER_READ | MP_OWNER_WRITE, (void **)&dm, &dm_head)) {
		printf("%s: failed creating shared memory %s\n",argv[0],dm_namep);
		ok_dm=0;
	} else ok_dm=1;
	
	/* init semaphores */
	if (ok_dm) {
		printf("%s: creating semaphores...\n",argv[0]);
		for (ok_sema=0;ok_sema<NR_OF_BUFFERS;ok_sema++) {
			if (_os_sema_init( &(dm->buf_sema[ok_sema]) )) {
				printf("%s: failed to init semaphore no. %d\n",argv[0],ok_sema);
			}
		}
	}
	
	/* create transmitter pipe */
	if (ok_dm && ok_sema==NR_OF_BUFFERS) {
		printf("%s: creating transmitter pipe %s...\n",argv[0], TRANS_PIPE_NAME);
		if (_os9_create( TRANS_PIPE_NAME,WRITE_|READ_,&tpipe,MP_OWNER_READ|MP_OWNER_WRITE)) {
			printf("%s: failed creating transmitter pipe %s\n",argv[0],TRANS_PIPE_NAME);
			ok_tpipe=0;
		}
		else ok_tpipe=1;
	}
	
	/* create writer pipes */
	/* start writer 0..N-1 */
	if (ok_dm && ok_sema==NR_OF_BUFFERS && ok_tpipe) {
		nofpipes=0;
		type_lang=mktypelang(MT_PROGRAM, ML_OBJECT);
		argchild[0]="writer";
		argchild[1]=BUF_MOD_NAME;
		argchild[3]=NULL;
		argchild[4]=NULL;
		while (nofpipes<nofwri) {
			printf("%s: creating pipe and writer for %d...\n",argv[0],nofpipes);
			sprintf(pipename,WRITER_PIPE_NAME,nofpipes);
			if(_os9_create( pipename,WRITE_|READ_,&wpipes[nofpipes],MP_OWNER_READ|MP_OWNER_WRITE)) {
				printf("%s: failed opening pipe %s\n",argv[0],pipename);
				break;
			}
			ok_wpipes++;
			argchild[2]=pipename;
			if (_os_exec(_os_fork, 10, 3, argchild[0], argchild, _environ, 0, &pids[nofpipes], type_lang, orphan )) {
				printf("%s: failed forking child %d\n",argv[0],nofpipes);
				_os_close(wpipes[nofpipes]);
				break;
			}
			nofpipes++;
			ok_writers++;
		}
	}

	/* start receiver */				
	if (ok_dm && ok_sema==NR_OF_BUFFERS && ok_tpipe && ok_writers==nofwri) {
		char t[4];
		printf("%s: creating receiver...\n",argv[0]);
		sprintf(t,"%d",ok_writers);
		argchild[0] = "receiver";
		argchild[1] = BUF_MOD_NAME;
		argchild[2] = TRANS_PIPE_NAME;
		argchild[3] = t;
		argchild[4] = NULL;
		if (_os_exec(_os_fork, 20, 3, argchild[0], argchild, _environ, 0, &pids[nofpipes], type_lang, orphan )) {
			printf("%s: failed forking child %s\n",argv[0],argchild[0]);
			ok_receiver=0;
		} else ok_receiver=1;
	}
	
	/* start transmitter */
	if (ok_dm && ok_sema==NR_OF_BUFFERS && ok_tpipe && ok_writers==nofwri && ok_receiver) {
		printf("%s: creating transmitter...\n",argv[0]);
		argchild[0] = "transmitter50275";
		argchild[1] = NULL;
		argchild[2] = NULL;
		argchild[3] = NULL;
		argchild[4] = NULL;
		if (_os_exec(_os_fork, 30, 3, argchild[0], argchild, _environ, 0, &pids[MAX_PROCS+1], type_lang, orphan )) {
			printf("%s: failed forking child %s\n",argv[0],argchild[0]);
			ok_transmitter=0;
		} else ok_transmitter=1;
	} 
	if (ok_receiver && !ok_transmitter) {
		printf("%s: no transmitter present, so sending EOT\n",argv[0]);
		argchild[0]=END_OF_TRANS;
		count=sizeof(dest);
		dest=0;
		_os_write(tpipe,&dest,&count);
		count=sizeof(dest);
		dest=strlen(argchild[0])+1;
		_os_write(tpipe,&dest,&count);
		count=dest;
		_os_write(tpipe,argchild[0],&count);
	}
		
	
	/* wait for nofpipes+2 terms */
	printf("%s: waiting for %d child processes to terminate...\n", argv[0], ok_transmitter+ok_receiver+ok_writers);

	if (ok_transmitter) {
		_os_wait(&pids[MAX_PROCS+1], &retcode);
	}
	
	if (ok_receiver) {
		_os_wait(&pids[MAX_PROCS+0], &retcode);
	}
	
	while (ok_writers) {
		_os_wait(&pids[ok_writers--], &retcode);
	}

	/* close writer pipes */
	printf("%s: closing %d writer pipes...\n", argv[0], ok_wpipes);
	while (ok_wpipes) {
		if (_os_close(wpipes[ok_wpipes-1]) )
			printf("%s: failed closing pipe %d\n", argv[0], ok_wpipes);
		ok_wpipes--;
	}

	/* close transmitter pipe */
	if (ok_tpipe) {
		printf("%s: closing transmitter pipe...\n", argv[0]);
		if (_os_close(tpipe))
			printf("%s: failed closing transmitter pipe\n",argv[0]);
		else ok_tpipe=0;
	}
	
	/* release semaphores */
	printf("%s: terminating %d semaphores...\n", argv[0], ok_sema);
	while (ok_sema) {
		if (_os_sema_term( &(dm->buf_sema[--ok_sema]) ))
			printf("%s: failed terminating semaphore no. %d\n",argv[0],ok_sema);
	}
	
	/* unlink from data module */
	if (ok_dm) {
		printf("%s: unlinking shared data module...\n", argv[0]);
		if (_os_unlink((mh_com*)dm_head))
			printf("%s: failed unlinking shared data module\n",argv[0]);
		else ok_dm=0;
	}
	_os_exit(0);
}
