/* receiver.c
 *
 * Program reads a message from transmitterpipe, timestamps it, writes it to a 
 * memory area and sends message to appropriate writerprocess. When a EOT is received
 * it sends shutdown message to subprocesses (writers) and shuts down itself.
 */
 
#include <stdio.h>
#include <process.h>
#include <time.h>
#include <alarm.h>
#include <signal.h>
#include <types.h>
#include <stdlib.h>
#include <dexec.h>
#include <modes.h>
#include <sysio.h>
#include <cglob.h>
#include <string.h>
#include "netdefs.h"

struct gtime_t { 
	u_int8 reserved;
	u_int8 hour;
	u_int8 min;
	u_int8 sec; 
};

int main(int argc, char *argv[]) {
	/* aux vars */
	u_int32 count;
	int lp1;
	int eot;
	/* writer side comm */
	int32 nofw;
	char pipename[64];
	path_id pipes[MAX_PROCS];
	/* data module vars */
	char dm_name[64];
	char *dm_namep = dm_name;
	u_int16 attr_rev, type_lang;
	mh_com *dm_head;
	BufModType *dm;
	/* transmitter side comm */
	path_id tpipe;
	struct { u_int32 dest, len; } packet;
	/* buffer handling */
	u_int32 time;
	u_int32 date;
	u_int16 day;
	u_int32 ticks; 
	u_int32 free_buffer;
	char inbuf[MAX_MSG_LENGTH];
	/* resource management */
	char ok_dm = 0;
	char ok_tpipe = 0;
	char ok_wpipes = 0;
	
	
	
	
	if (argc < 4) {
		printf("Usage: %s <datamodule> <transmitter-pipe> <no-of-writers>\n", argv[0]);
		_os_exit(1);
	}
	nofw = atoi(argv[3]);
	
	printf("%s: starting\n", argv[0]);
	
	/* open data module */
	attr_rev = mkattrevs(MA_REENT, 0);
	type_lang = mktypelang(MT_ANY, ML_ANY);
	strcpy(dm_namep, argv[1]);
	printf("%s: attempting to link to %s...\n", argv[0], dm_namep);
	if (_os_link(&dm_namep, &dm_head, (void**)&dm, &type_lang, &attr_rev)) {
		printf("%s: failed linking to %s\n", argv[0],dm_namep);
		ok_dm=0;
	} else {
		ok_dm=1;
		printf("%s: linked to %s\n", argv[0], dm_namep);
	}
	
	/* open transmitter pipe */
	if (ok_dm) {
		printf("%s: opening transmitter pipe %s...\n", argv[0], argv[2]);
		if (_os_open(argv[2],READ_,&tpipe)) {
			printf("%s: opening transmitter pipe %s failed\n", argv[0], argv[2]);
			ok_tpipe=0;
		} else {
			printf("%s: transmitter pipe %s ok\n", argv[0], argv[2]);
			ok_tpipe=1;
		}
	}
	
	if (ok_dm && ok_tpipe) {
		printf("%s: opening %d writer pipes...\n", argv[0], nofw);
		/* open pipes (in loop) */
		for (lp1=0;lp1<nofw;lp1++) {
			sprintf(pipename,WRITER_PIPE_NAME,lp1);
			if (_os_open(pipename,READ_,&pipes[lp1])) {
				printf("%s: opening writer pipe %s failed\n", argv[0], pipename);
				break;
			} else {
				ok_wpipes++;
			}
		}
	}
	
	/* do */
	if (ok_dm && ok_tpipe && ok_wpipes==nofw) {
		do {
			/* read message from input pipe */
			/* timestamp message */
			count=sizeof(packet);
			_os_read(tpipe, &packet, &count);
			_os9_getime(0,&time,&date,&day,&ticks);
			_os_read(tpipe, &inbuf, &packet.len);
/*			printf("%s: got packet for %d, size=%d containing '%s'\n", argv[0], packet.dest, packet.len,inbuf); */
			/* look for EOT */
			eot=!strcmp(inbuf,END_OF_TRANS);
			/* verify that writer exists and not EOT received */
			if (!eot && packet.dest<nofw) {
				/* look for free buffer */
				/* put message in free buffer */
				for (free_buffer=0;free_buffer<NR_OF_BUFFERS;free_buffer++)
					if (!dm->buf_used[free_buffer]) {
						dm->time_stamp[free_buffer]=time;
						strcpy(dm->msg_buf[free_buffer],inbuf);
						/* update used-flag */
						_os_sema_p(&dm->buf_sema[free_buffer]);
						dm->buf_used[free_buffer]=1;
						_os_sema_v(&dm->buf_sema[free_buffer]);
						break;
					}
				if (free_buffer==NR_OF_BUFFERS) {
					printf("%s: out of buffers...message discarded at %x\n",argv[0],time);
				} else {
					/* pass message to writer */
					count=sizeof(free_buffer);
					_os_write(pipes[packet.dest],&free_buffer,&count);
				}
			} else if (!eot) {
				printf("%s: packet to %d unsent> '%s'\n",argv[0],packet.dest,inbuf);
			}
			/* while not end-of-transmission */
		} while (!eot);
	}
		
	/* close pipes */
	printf("%s: transmitting EOT and closing %d writer pipes...\n", argv[0], ok_wpipes);
	while (ok_wpipes) {
		--ok_wpipes;
		free_buffer=-1;
		count=sizeof(free_buffer);
		if (_os_write(pipes[ok_wpipes],&free_buffer,&count)) 
			printf("%s: error writing EOT to pipe no. %d\n", argv[0], ok_wpipes);
		if (_os_close(pipes[ok_wpipes])) 
			printf("%s: closing pipe no. %d failed\n", argv[0], ok_wpipes);
	}
	
	if (ok_tpipe) {
		printf("%s: closing transmitter pipe %s\n", argv[0], argv[2]);			
		_os_close(tpipe);
	}
	
	if (ok_dm) {
		/* unlink from data module */
		printf("%s: unlinking data module...\n", argv[0]);
		_os_unlink(dm_head);
	}
		
	_os_exit(0);
}
