/* writer.c
 *
 * Program reads a message and timestamp from a messagebuffer and outputs it to
 * screen as long as no stoprequest is recieved.
 *
 */
 
#include <stdio.h>
#include <process.h>
#include <time.h>
#include <alarm.h>
#include <signal.h>
#include <types.h>
#include <stdlib.h>
#include <dexec.h>
#include <modes.h>
#include <sysio.h>
#include <cglob.h>
#include <string.h>
#include "netdefs.h"

struct gtime_t { 
	u_int8 reserved;
	u_int8 hour;
	u_int8 min;
	u_int8 sec; 
};

int main(int argc, char *argv[]) {
	u_int32 count;
	int bfn;
	path_id pipe;
	struct gtime_t tstamp;
	char localbuffer[MAX_MSG_LENGTH];
	char dm_name[64];
	char *dm_namep = dm_name;
	mh_com *dm_head;
	BufModType *dm;
	u_int16 attr_rev, type_lang;
	int wnum;
	
	if (argc < 3) {
		printf("Usage: %s <datamodule> <pipe>\n", argv[0]);
		_os_exit(1);
	}
	
	sscanf(argv[2],WRITER_PIPE_NAME,&wnum);
	
	printf("%s %s: starting...\n", argv[0], argv[2]);
	/* open pipe */
	if (!_os_open(argv[2],READ_,&pipe)) {
		/* get access to shared memory module */
		attr_rev = mkattrevs(MA_REENT, 0);
		type_lang = mktypelang(MT_ANY, ML_ANY);
		strcpy(dm_namep, argv[1]);
/*		printf("%s %s: attempting to link to %s\n", argv[0], argv[2], dm_namep);*/
		if (!_os_link(&dm_namep, &dm_head, (void**)&dm, &type_lang, &attr_rev)) {
/*			printf("%s %s: linked to %s\n", argv[0], argv[2], dm_namep);*/
			/* repeat: */
			do {
				/* read the number of a buffer to output from pipe */
				count = sizeof(bfn);
				_os_read(pipe, &bfn, &count);
				/* if number is negative - break loop */
				if (bfn<0) break;
				/* read msg from the buffer into local buffer */
				strcpy( localbuffer, dm->msg_buf[bfn] );
				tstamp.sec = (dm->time_stamp[bfn]>>0) &0xff;
				tstamp.min = (dm->time_stamp[bfn]>>8) & 0xff;
				tstamp.hour = (dm->time_stamp[bfn]>>16) & 0xff;
				/* lock the buffer, clear the flag, release the buffer */
				_os_sema_p( &(dm->buf_sema[bfn]) );
				dm->buf_used[bfn]=0;
				_os_sema_v( &(dm->buf_sema[bfn]) );
				/* output messsage from local buffer */
/*				printf("%s %s (%02d:%02d:%02d): %s\n",argv[0],argv[2],tstamp.hour,tstamp.min,tstamp.sec,localbuffer);*/
				printf("w%d (%02d:%02d:%02d): %s\n",wnum,tstamp.hour,tstamp.min,tstamp.sec,localbuffer);
				
			} while (bfn>=0);
/*			printf("%s %s: unlinking memory and closing pipe\n",argv[0],argv[2]);*/
			_os_unlink(dm_head);
			_os_close(pipe);
		} else {
			printf("%s %s: failed to get access to datamodule '%s', closing pipe\n",argv[0],argv[2],argv[1]);
			_os_close(pipe);
		}
	} else {
		printf("%s %s: failed to open pipe\n",argv[0],argv[2]);
	}
/*	printf("%s %s: exited...\n",argv[0],argv[2]);*/
	_os_exit(0);
}
