/* File: server.c
 *
 * This server accepts incoming connection on specified port (default:50275) and accepts usercommands
 * Exits upon receiving the command: stop_server. Continues to run when users logout.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <inet/in.h>
#include <inet/socket.h>
#include <iau_sock.h>
#include <alarm.h>
#include <types.h>
#include <time.h>

#define BUFLEN 500
#define WAKEUP_CODE 1

struct gtime_t { 
	u_int8 reserved;
	u_int8 hour;
	u_int8 min;
	u_int8 sec; 
};

struct gdate_t { 
	u_int16 year; 
	u_int8 month;
	u_int8 day;
};

void main(int argc, char *argv[]) {
	alarm_id id;
	u_int32 ticks;
	u_int16 day;
	u_int32 gtticks;
	struct gtime_t gtime;
	struct gdate_t gdate;
	int i, s, ls, status, port, addr_len, len, no_iter = 10;
	unsigned int tdelay = 1;
	struct	sockaddr_in	name, from;
	char	buffer[BUFLEN];
	alarm_id aid;
	char sample=0, stop=0, logoff=0;
	
	if (argc == 1) port = 50275;
	else
	if (argc == 2) {
		port = atoi(argv[1]);
	} else {
		printf("Usage: %s [portno]\n",argv[0]);
		exit(1);
	}

	/* Create the socket */
	ls = socket(AF_INET, SOCK_STREAM, 0);
    if ( ls == -1 )
    {
      	perror(strerror(errno));
      	fprintf(stderr,"%s: Can not make listen socket\n",argv[0]);
      	exit(errno);
    }
	
	setsockopt(ls, SOL_SOCKET, SO_REUSEADDR, 0, 0);
	
	name.sin_family = AF_INET;
	name.sin_port	= port;
	name.sin_addr.s_addr = INADDR_ANY;
	
	/* Bind socket address to the socket  */
    status = bind(ls, (struct sockaddr*) &name, sizeof(name));
    if ( status == -1 )
    {
      	perror(strerror(errno));
        fprintf(stderr," Can not bind the address to the socket\n");
        exit(errno);
    }

    /* Make cue where the client processes can ask for connection */
    status = listen(ls, 1);
    if ( status == -1 )
    {
      	perror(strerror(errno));
        exit(errno);
    }

	printf("%s: ready listning on port %d\n", argv[0], port); 

	do {
	    /* Waits on a connection from a client and returns with
		 * the address on the client process ("peer") and a new
		 * socket decriptor to the new connection  
		 */
		printf("%s: waiting for clients...\n",argv[0]);
		addr_len = sizeof(struct sockaddr_in);
		s = accept(ls, (struct sockaddr*) &from, &addr_len);
		if ( s == -1 ) {
			perror(strerror(errno));
			exit(errno);
		} else {
			ticks = from.sin_addr.s_addr;
			printf("%s: connect from %d.%d.%d.%d...\n",argv[0],(ticks>>24)&0xff,(ticks>>16)&0xff,(ticks>>8)&0xff,ticks&0xff);
		}
		
		/* Set socket to be non-blocking */
		sock_block(s, 1);
		
		/* Send the data */
		len=sprintf(buffer, "\x1bH\x1bJ************************************\xA Welcome s to this CoolLine Server\xANo leaching and please don't upload!\xA************************************\xA\xA");
		send( s, buffer, len, 0); 
		buffer[0]=0;
		logoff=0;
		stop=0;
		sample=0;
		i=0;
		_os_alarm_cycle(&aid, WAKEUP_CODE, CLOCKS_PER_SEC);
		printf("%s: starting command loop...\n",argv[0]);
		do {
			if (sample) {
				len = sprintf(buffer, "\x1b&a%iy%iChere is i = %d",10,1,i);
				send( s, buffer, len, 0);
			}
			/* send system time */
			_os9_getime(0, (u_int32*)&gtime, (u_int32*)&gdate, &day, &gtticks );			
			len=sprintf(buffer,"\x1b&a%iy%iC%04d-%02d-%02d %02d:%02d:%02d\n\x1b&a%iy%iC",1,60,gdate.year,gdate.month,gdate.day,gtime.hour,gtime.min,gtime.sec,20,1);
			send( s, buffer, len, 0);

			/* get command if present */
			len=sizeof(buffer)-1;
			len=recv( s, buffer, len, 0 );
			if (len>2) {
				/* remove CR and LF chars */
				buffer[len]=0; buffer[len-1]=0; buffer[len-2]=0;
/*				printf("%s: received %d bytes = '%s'...\n",argv[0], len, buffer);*/
				/* recognize command */
				if (!strcmp(buffer,"bye")) logoff=1;
				else if (!strcmp(buffer,"stop")) sample=0;
				else if (!strcmp(buffer,"start")) sample=1;
				else if (!strcmp(buffer,"stop_server")) { sample=0; logoff=1; stop=1; }
				else if (!strncmp(buffer,"set",3)) { 
					_os_alarm_delete(aid);
					ticks=atoi(&buffer[3]) | 0x80000000;
					_os_alarm_cycle(&aid, WAKEUP_CODE, ticks);
				} else {
					len=sprintf(buffer,"\x1b&a%iy%iCWHAT????? say again...",21,1);
					send( s, buffer, len, 0);
				}
			}
			/*printf("%s: prepping to sleep (sample=%d,stop=%d,logoff=%d)...\n",argv[0],sample,stop,logoff);*/
			ticks=0;
			i++;
			if (!logoff && !stop) _os9_sleep(&ticks);
		} while (!logoff);
		_os_alarm_delete(aid);
		len=sprintf(buffer, "\x1bH\x1bJ**************************************\xaThank you for using this superb server\xathat rulez the world of nOS/9\xA**************************************\xA\xA");
		send( s, buffer, len, 0); 
		
		close(s);
	} while (!stop);
	printf("%s: server stopped, closing socket...\n", argv[0]);
	close(ls);
	exit(0);
}
